%% 2014 Alexey Ryabov
%% This function runs the metacommunity model
%% The function returns TSpan - time vector, N_Dyn (L, L, sp, TSpan) -- the dynamics
%% of species. R_Dyn(L, L, res, TSpan) the dynamics of the resources

function [TSpan, N_Dyn, R_Dyn] = run_MetaCom...
    (N,  c,   R_eq,    r,    m, d, DispRes,...
    R,     S,    D,    TSpan, fl, dx, L,ReshuffleTime,ReshuffleStep)
% init sp distr, sp cons rates,  R*,growth rate, mort,diffusiv,
% init res distr, suppl points,  Suppl rates,
% TimeSpan, flux, dx, L


%% Find an appropriate time step
Diff = max(d(:));
if Diff == 0
    dt = 0.1;
else
    dt = min(0.1, 0.5 * (dx^2/(4*Diff))); %time step should satisfy the stability criterion
end
CurrentTime = 0;

tsteps = length(TSpan(1):dt:TSpan(end));
[K,x,y] = SpatInitialize(R_eq, r,  squeeze(m(1, 1, :)), tsteps,dt,L,dx);


%% Initialyze the output aurgments
N_Dyn = zeros([size(N), length(TSpan)]);
R_Dyn = zeros([size(R), length(TSpan)]);

%% Save initial conditions
N_Dyn(:, :, :, 1) = N;
R_Dyn(:, :, :, 1) = R;
t = 0;

for ti=2:length(TSpan);
    TimeStep = dt;
    while t < TSpan(ti)
        t = t + TimeStep;
        if t > TSpan(ti)
            t = TSpan(ti);
            TimeStep = dt - (t - TSpan(ti));
        end
        %% Run the model
        [f_N, f_R] = SpatLocalChange(R, D, S, N, r, K, m, c);
        [N_rr] = SpatBoundary(fl, N); % boundary conditions preparation for N
        [N_Laplace] = Laplace2D_2x2(N_rr, dx); % Laplace operator
        for si=1:size(N, 3)
            dN(:, :, si) = f_N(:, :, si) + d(si) * N_Laplace(2:end-1, 2:end-1, si);
        end
        
        %Use to have diffusion of the resources
%         [R_rr] = SpatBoundary(fl,R); % optionally also for R
%         [R_Laplace] = Laplace2D_2x2(R_rr,dx);
%         for ri=1:res
%             dR(:, :, ri) = f_R(:, :, ri) + DispRes(ri) * R_Laplace(2:end-1,2:end-1, ri);
%         end
        dR = f_R; % differential equations for R1 and R2 (in this case both
        
        % Euler step for every iteration
        N = N + TimeStep*dN;
        R = R + TimeStep*dR;
        CurrentTime = CurrentTime + TimeStep;
        if t == TSpan(ti)
            N_Dyn(:, :, :, ti) = N;
            R_Dyn(:, :, :, ti) = R;
            break;
        end
%         %Reshuffle
%         if t > ReshuffleTime
%             S = resourceshuffle(S,L,size(R,3));
%             ReshuffleTime = ReshuffleTime + ReshuffleStep;
%         end
    end
end


end



